"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const redis = require("redis");
const bluebird = require("bluebird");
class RedisClient {
    constructor(options) {
        this.client = redis.createClient(options);
        bluebird.promisifyAll(redis);
    }
    closeConnection() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.client.end(false);
        });
    }
    flushdb() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.client.flushdbAsync();
        });
    }
    hgetall(key) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.client.hgetallAsync(key);
        });
    }
    smembers(key) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.client.smembersAsync(key);
        });
    }
    hdel(key, field) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.client.hdelAsync(key, field);
        });
    }
    srem(key, member) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.client.sremAsync(key, member);
        });
    }
    zrem(key, member) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.client.zremAsync(key, member);
        });
    }
    set(key, value) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.client.setAsync(key, value);
        });
    }
    zadd(key, score, member) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.client.zaddAsync(key, score, member);
        });
    }
    sadd(key, member) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.client.saddAsync(key, member);
        });
    }
    hmset(key, values) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.client.hmsetAsync(key, values);
        });
    }
    rpush(key, element) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.client.rpushAsync(key, element);
        });
    }
    llen(key) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.client.llenAsync(key);
        });
    }
    sscan(key, cursor, pattern) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield this.client.sscanAsync(key, cursor, 'MATCH', pattern);
            return {
                cursor: Number(response[0]),
                data: response[1],
            };
        });
    }
    sinter(keys) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.client.sinterAsync(keys);
        });
    }
    sunion(keys) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.client.sunionAsync(keys);
        });
    }
    zrangebyscore(key, min, max, offset, count) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.client.zrangebyscoreAsync(key, min, max, 'LIMIT', offset, count);
        });
    }
    zrange(key, start, stop) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.client.zrangeAsync(key, start, stop);
        });
    }
    zrevrange(key, start, stop) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.client.zrevrangeAsync(key, start, stop);
        });
    }
    lrange(key, start, stop) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.client.lrangeAsync(key, start, stop);
        });
    }
    lrem(key, count, element) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.client.lremAsync(key, count, element);
        });
    }
    del(key) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.client.delAsync(key);
        });
    }
    get(key) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.client.getAsync(key);
        });
    }
    hmget(key, properties) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.client.hmgetAsync(key, properties);
        });
    }
    eval(...args) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.client.evalAsync(args);
        });
    }
}
exports.RedisClient = RedisClient;
